"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = __importDefault(require("process"));
const axios_1 = __importDefault(require("axios"));
const child_process_promise_1 = require("child-process-promise");
const commander_1 = __importDefault(require("commander"));
const inquirer_1 = __importDefault(require("inquirer"));
const package_json_1 = __importDefault(require("../../package.json"));
const config_1 = __importDefault(require("../config"));
const gcloud_1 = __importDefault(require("../gcloud"));
const keymanager_1 = require("../keymanager");
const GITHUB_API_BASE = 'https://api.github.com';
function makeGithubRequest(uri, body, token, method = 'POST') {
    return __awaiter(this, void 0, void 0, function* () {
        return axios_1.default({
            method,
            url: uri,
            data: body,
            headers: {
                'User-Agent': 'cloudbuild-github',
            },
            auth: {
                username: 'token',
                password: token,
            },
        });
    });
}
function deploy() {
    return __awaiter(this, void 0, void 0, function* () {
        const ui = new inquirer_1.default.ui.BottomBar();
        const webhookSecret = yield keymanager_1.keyManager.getWebhookSecret();
        const google = yield gcloud_1.default();
        const projectId = yield google.auth.getProjectId();
        const memory = commander_1.default.memory;
        ui.log.write(`Deploying cloud functions for project ${projectId}.`);
        if (commander_1.default.delete) {
            try {
                yield child_process_promise_1.spawn('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubWebhook'], { stdio: 'inherit' });
            }
            catch (err) {
                ui.log.write('Could not delete cloudbuildGithubWebhook, skipping.');
            }
            try {
                yield child_process_promise_1.spawn('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubNotifier'], { stdio: 'inherit' });
            }
            catch (err) {
                ui.log.write('Could not delete cloudbuildGithubNotifier, skipping.');
            }
        }
        yield child_process_promise_1.spawn('gcloud', [
            'beta',
            'functions',
            'deploy',
            'cloudbuildGithubWebhook',
            '--trigger-http',
            '--runtime=nodejs10',
            `--memory=${memory}`,
        ], { stdio: 'inherit' });
        yield child_process_promise_1.spawn('gcloud', [
            'beta',
            'functions',
            'deploy',
            'cloudbuildGithubNotifier',
            '--trigger-topic',
            'cloud-builds',
            '--runtime=nodejs10',
            `--memory=${memory}`,
        ], { stdio: 'inherit' });
        const webhookUrl = `https://us-central1-${projectId}.cloudfunctions.net/cloudbuildGithubWebhook`;
        yield Promise.all(Object.keys(config_1.default.repos).map((repoName) => __awaiter(this, void 0, void 0, function* () {
            const repoToken = yield keymanager_1.keyManager.getGithubToken(repoName);
            ui.log.write(`Setting up repository webhook for ${repoName}`);
            const hooksUri = `${GITHUB_API_BASE}/repos/${repoName}/hooks`;
            const existingHooks = yield makeGithubRequest(hooksUri, null, repoToken, 'GET');
            yield Promise.all(existingHooks.data
                .filter((h) => h.config.url === webhookUrl)
                .map((h) => __awaiter(this, void 0, void 0, function* () { return makeGithubRequest(`${hooksUri}/${h.id}`, null, repoToken, 'DELETE'); })));
            const hook = {
                name: 'web',
                config: {
                    url: webhookUrl,
                    // eslint-disable-next-line @typescript-eslint/camelcase
                    content_type: 'json',
                    secret: webhookSecret,
                },
                events: ['pull_request'],
            };
            return makeGithubRequest(hooksUri, hook, repoToken);
        })));
        ui.log.write('Done!');
    });
}
commander_1.default
    .version(package_json_1.default.version)
    .option('--delete', 'Delete existing functions first')
    .option('--memory [size]', 'Memory usage for function', '256MB')
    .parse(process_1.default.argv);
deploy().then(() => process_1.default.exit(), (err) => {
    console.error('Unexpected error.', err);
    process_1.default.exit(1);
});
//# sourceMappingURL=cloudbuild-cli-deploy.js.map