"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2017 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const webhooks_1 = __importDefault(require("@octokit/webhooks"));
const http_status_codes_1 = __importDefault(require("http-status-codes"));
const parse_duration_1 = __importDefault(require("parse-duration"));
const gcloud_1 = __importDefault(require("./gcloud"));
const keymanager_1 = require("./keymanager");
const config_1 = __importDefault(require("./config"));
const constants_1 = require("./constants");
const MILLIS_IN_SECOND = 1000;
function handlePullRequest(event) {
    return __awaiter(this, void 0, void 0, function* () {
        if (event.action !== 'opened' && event.action !== 'synchronize') {
            console.log('Unhandled event action: ', event.action);
            return;
        }
        const pull = event.pull_request;
        const otherBranchRemoteRef = `refs/pull/${pull.number}/head`;
        const otherBranchLocalRef = `refs/remotes/origin/PR-${pull.number}`;
        const baseBranchRemoteRef = `refs/heads/${pull.base.ref}`;
        const baseBranchLocalRef = `refs/remotes/origin/${pull.base.ref}`;
        const repo = event.repository.full_name;
        const substitutions = {
            _REPOSITORY_URL: pull.base.repo.clone_url,
            _OTHER_BRANCH_REMOTE_REF: otherBranchRemoteRef,
            _OTHER_BRANCH_LOCAL_REF: otherBranchLocalRef,
            _BASE_BRANCH_REMOTE_REF: baseBranchRemoteRef,
            _BASE_BRANCH_LOCAL_REF: baseBranchLocalRef,
            [constants_1.STATUSES_URL_KEY]: pull.statuses_url,
            [constants_1.COMMENTS_URL_KEY]: pull.comments_url,
        };
        const prTag = `pr.${pull.number}`;
        const tags = [
            `repo.${event.repository.name.replace('_', '-')}`,
            `sender.${event.sender.login}`,
            prTag,
        ];
        const google = yield gcloud_1.default();
        const projectId = yield google.auth.getProjectId();
        const cloudbuild = google.cloudbuild({ version: 'v1' });
        const existingBuilds = yield cloudbuild.projects.builds.list({
            projectId,
            filter: `tags="${prTag}"`,
        });
        existingBuilds
            .data.builds.filter((build) => build.status === 'QUEUED' || build.status === 'WORKING')
            .forEach((build) => {
            console.log(`Found existing build ${build.id}. Cancelling.`);
            cloudbuild.projects.builds.cancel({ projectId, id: build.id });
        });
        console.log(`Starting cloud build for pull request ${pull.number}.`);
        const cloudbuildConfig = config_1.default.repos[repo].cloudbuild;
        const sanitizedConfig = cloudbuildConfig.timeout
            ? Object.assign({}, cloudbuildConfig, { timeout: `${parse_duration_1.default(cloudbuildConfig.timeout) /
                    MILLIS_IN_SECOND}s` }) : cloudbuildConfig;
        yield cloudbuild.projects.builds.create({
            projectId,
            requestBody: Object.assign({}, sanitizedConfig, { options: Object.assign({}, sanitizedConfig.options, { substitutionOption: 'ALLOW_LOOSE' }), substitutions,
                tags }),
        });
    });
}
function handleWebhook(req, res) {
    return __awaiter(this, void 0, void 0, function* () {
        const secret = yield keymanager_1.keyManager.getWebhookSecret();
        const webhooks = new webhooks_1.default({
            secret,
        });
        if (!webhooks.verify(req.rawBody.toString(), req.get('X-Hub-Signature'))) {
            console.error('Invalid signature.');
            res.status(http_status_codes_1.default.BAD_REQUEST).end();
            return;
        }
        const event = req.body;
        const eventType = req.get('X-GitHub-Event');
        switch (eventType) {
            case 'pull_request':
                yield handlePullRequest(event);
                break;
            default:
                console.log('Unhandled event type: ', eventType);
                break;
        }
        res.status(http_status_codes_1.default.OK).end();
    });
}
exports.default = handleWebhook;
//# sourceMappingURL=webhook.js.map